package com.dino.scrum.sysmag.controller;

import com.dino.scrum.sysmag.model.Product;
import com.dino.scrum.sysmag.model.dto.IdDto;
import com.dino.scrum.sysmag.model.dto.QuantityChange;
import com.dino.scrum.sysmag.service.ProductServiceImpl;
import com.dino.scrum.sysmag.validationGroup.UpdateGroup;
import com.dino.scrum.sysmag.validator.QuantityChangeValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PageableDefault;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Map;

/**
 * Created by prgres on 2019-01-12.
 */

@RestController
@RequestMapping(value = "/api")
public class ProductController {

    private final
    ProductServiceImpl productService;
    private final
    QuantityChangeValidator quantityChangeValidator;

    @Autowired
    public ProductController(ProductServiceImpl productService, QuantityChangeValidator quantityChangeValidator) {
        this.productService = productService;
        this.quantityChangeValidator = quantityChangeValidator;
    }

    @GetMapping(value = "/get-all")
    public Iterable<Product> getAll(@PageableDefault(size = Integer.MAX_VALUE) Pageable pageable) {
        System.out.println(pageable.getPageSize());

        try {
            return productService.getAll(pageable);
        } catch (Exception e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.toString());
        }
    }

    @GetMapping(value = "/get-price-of-all")
    public Map getPriceOfAllProducts() {
        return Collections.singletonMap("price-of-all", productService.getPriceOfAll());
    }

    @PostMapping(value = "/product/add")
    public ResponseEntity addProduct(@Valid @RequestBody Product product) {
        try {
            productService.add(product);
            return new ResponseEntity(product, HttpStatus.CREATED);
        } catch (Exception e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.toString());
        }
    }

    @DeleteMapping(value = "/product/delete")
    public ResponseEntity deleteProduct(@RequestBody IdDto id) {
        try {
            productService.delete(id.getId());
            return new ResponseEntity("Deleted" + id, HttpStatus.OK);
        } catch (Exception e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.toString());
        }
    }

    @GetMapping(value = "/product/get-by-id")
    public Product getById(@RequestBody IdDto id) {
        try {
            return productService.getById(id.getId());
        } catch (Exception e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.toString());
        }
    }

    @PostMapping(value = "/product/update/{id}")
    public Product update(@Validated(UpdateGroup.class) @RequestBody Product product, @PathVariable("id") long id) {
        try {
            return productService.update(product, id);
        } catch (Exception e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.toString());
        }
    }

    @PostMapping(value = "/product/change-quantity")
    public Product changeQuantity(@RequestBody QuantityChange quantityChange) {
        try {
            quantityChangeValidator.validate(quantityChange);
            return productService.changeQuantity(quantityChange);
        } catch (Exception e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.toString());
        }
    }
}