package com.dino.scrum.sysmag.service;

import com.dino.scrum.sysmag.model.Product;
import com.dino.scrum.sysmag.model.dto.QuantityChange;
import com.dino.scrum.sysmag.repository.ProductRepository;
import com.dino.scrum.sysmag.validator.ProductValidator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;

/**
 * Created by prgres on 2019-01-12.
 */

@Service
public class ProductServiceImpl implements ProductService {

    private final
    ProductRepository productRepository;

    private final
    ProductValidator productValidator;

    @Autowired
    public ProductServiceImpl(ProductRepository productRepository, ProductValidator productValidator) {
        this.productRepository = productRepository;
        this.productValidator = productValidator;
    }

    @Override
    public Page<Product> getAll(Pageable pageable) throws Exception {
        System.out.println("pagenumber: " + pageable.getPageNumber() + " pageSize: " + pageable.getPageSize() + " sort: " + pageable.getSort());

        if (pageable.getSort().isUnsorted()) {
            pageable = PageRequest.of(pageable.getPageNumber(), pageable.getPageSize(), new Sort(Sort.Direction.ASC, "id"));
        }

        Page<Product> productSlice = productRepository.findAll(pageable);

        productValidator.checkIfPageIsOutOfBound(productSlice.getTotalPages(), pageable.getPageNumber());

        return productSlice;
    }

    @Override
    public Product changeQuantity(QuantityChange quantityChange) throws Exception {
        productValidator.checkIfNotExists(quantityChange.getId());

        productRepository.save(
                productRepository.findById(quantityChange.getId())
                        .setChangeQuantity(quantityChange.getChange())
        );

        return productRepository.findById(quantityChange.getId());
    }

    @Override
    public Product getById(Long id) throws Exception {
        productValidator.checkIfNotExists(id);
        return productRepository.findById(id.longValue());
    }

    @Override
    public float getPriceOfAll() {
        BigDecimal result = new BigDecimal(0);
        Iterable<Product> tempProductList = productRepository.findAll();

        for (Product product : tempProductList) {
            result = result.add(
                    product.getPrice()
                            .multiply(new BigDecimal(product.getQuantity())));
        }
        return result.floatValue();
    }

    @Override
    public Product add(Product product) throws Exception {
        productValidator.checkIfNotExists(product.getName());
        return productRepository.save(product);
    }

    @Override
    public Product update(Product productReceived, long id) throws Exception {
        productValidator.checkIfNotExists(productReceived.getId());

        Product productToChange = productRepository.findById(id);

        if (productReceived.getName() != null)
            productToChange.setName(productReceived.getName());

        if (productReceived.getPrice() != null)
            productToChange.setPrice(productReceived.getPrice());

        if (productReceived.getQuantity() != null)
            productToChange.setQuantity(productReceived.getQuantity());

        if (productReceived.getQuantityMax() != null)
            productToChange.setQuantityMax(productReceived.getQuantityMax());

        if (productReceived.getImageLink() != null)
            productToChange.setImageLink(productReceived.getImageLink());

        productRepository.save(productToChange);

        return productRepository.findById(productToChange.getId().longValue());
    }


    @Override
    public long count() {
        return productRepository.count();
    }

    @Override
    public void delete(long id) throws Exception {
        productValidator.checkIfNotExists(id);
        productRepository.deleteById(id);
    }
}
